package com.print;

import java.io.IOException;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.fop.apps.Fop;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.apps.MimeConstants;
import org.apache.fop.servlet.ServletContextURIResolver;

/**
 * Ein Servlet für Printausgaben, das FO-File wird dabei von einer URL geholt.
 * 
 * Voraussetzung: die Xalan Bibiotheken und Apache FOP Bibliotheken vorhanden
 * (für die Entwicklung wird aber nur fop.jar gebraucht)
 */
public final class FO2PDFTransformer extends HttpServlet {

	private static final long serialVersionUID = 8024083971529273992L;

	private FopFactory fopFactory = null;
	
	private Transformer transformer = null;
	
	@Override
	public void init(ServletConfig config) throws ServletException {

		super.init(config);

		this.fopFactory = FopFactory.newInstance();

		ErrorListener errorListener = new TransformerErrorListener(config
				.getServletContext());

		TransformerFactory factory  = TransformerFactory.newInstance();
		factory.setErrorListener(errorListener);
		/*
		 * Beschaffe den Transformer.
		 * 
		 * Ein solcher Transformer darf zwar wiederverwendet werden (eventuell
		 * vorher reset() rufen), jedoch nicht parallel (von konkurrierenden
		 * Threads), nur sequenziell. Diese Wiederverwendung kann aber sehr
		 * attraktiv sein, da das Parsen einer XSL-Hierarchie unter Umständen
		 * sehr aufwändig sein kann und eigentlich nur einmal zu erfolgen hat.
		 * 
		 * Für Performanzoptimierung kann man beispielsweise einen Pool von
		 * Instanzen halten.
		 * 
		 * In einer anderen Sichtweise kann man einen Transformer als teure
		 * Ressource sehen und als Outbound Resource Adapter verpacken. Dann ist
		 * das Pooling vom Container unterstützt!
		 * 
		 * Dieser URI Resolver ist für den Einsatz in einer Servlet Umgebung
		 * gedacht und kann zB dort relative URIs auflösen. Man spart sich das
		 * Jonglieren mit RealPathes etc.
		 */
		URIResolver uriResolver = new ServletContextURIResolver(getServletContext());
		try {
			this.transformer = factory.newTransformer();
			this.transformer.setURIResolver(uriResolver);
		} catch (TransformerConfigurationException e) {
			throw new ServletException(e);
		}

	}

	@Override
	protected void doGet(final HttpServletRequest req,
			final HttpServletResponse response)

	throws ServletException, IOException {

		try {
			response.setContentType("application/pdf");
			Fop fop = fopFactory.newFop(MimeConstants.MIME_PDF, response
					.getOutputStream());
			Source src = new StreamSource(req.getParameter("fo"));
			Result res = new SAXResult(fop.getDefaultHandler());
			/**
			 * Der Transformer kann immer wieder verwendet werden,
			 * ist aber nicht threadsafe: 
			 */
			synchronized (this) {
				this.transformer.transform(src, res);
				this.transformer.clearParameters();
			}
			
		} catch (Exception ex) {
			throw new ServletException(ex);
		} 
	}
}